import requests
import numpy as np

import time
from PIL import Image
from io import BytesIO

class FrameOverlay:
    def __init__(self):
        self.session = requests.Session()
        self.frames = {}
        self.base_array = None
        

    def fetch_frames(self):
        print("⏳ Fetching frames in progress...")
        while len(self.frames) < 59:
            try:
                r = self.session.get(
                    "http://pov.challs.ulisse.ovh:8888/api/stream",
                    timeout=5
                )

                # Extract metadata and process

                index = int(r.headers["index"].split(" of ")[0])
                img = Image.open(BytesIO(r.content)).convert("RGB")
                arr = np.array(img)
                
                if index not in self.frames:
                    self.frames[index] = arr
                    self.process_overlay(arr)
                    
                    # Save progress every 5 frames
                    if len(self.frames) % 5 == 0:
                        self.save_progress()
                        
            except Exception as e:
                print(f"Error: {str(e)}")
                time.sleep(1)
                
        return Image.fromarray(self.base_array.astype(np.uint8))

    def process_overlay(self, new_arr):
        if self.base_array is None:
            self.base_array = new_arr.astype(np.float32)
        else:
            # Exponential decay moving average
            alpha = 0.9  # Weight of the previous frame
            self.base_array = self.base_array * alpha + new_arr * (1 - alpha)

    def save_progress(self):
        progress_img = Image.fromarray(self.base_array.astype(np.uint8))
        progress_img.save(f"progress_{len(self.frames)}.png")

    def save_result(self):
        final_img = Image.fromarray(self.base_array.astype(np.uint8))
        final_img.save("flag_revealed.png")
        print("✅ Full flag saved in flag_revealed.png")


if __name__ == "__main__":
    overlay = FrameOverlay()
    overlay.fetch_frames()
    overlay.save_result()
